/***************************************************************************
    file:           ksensorssetting.h
    description:    settings management
    begin:          09/06/2024
    Copyright:      Giuseppe Della Bianca<giusdbg@gmail.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kconfig.h>

#include "general.h"
#include "ksensorssetting.h"


bool ksensorsSetting::autoStartOpt = false;
bool ksensorsSetting::noDockOpt = false;
bool ksensorsSetting::showSplashOpt = false;
bool ksensorsSetting::chartTestOpt = false;

/***************************************************************************
    constructor
 ***************************************************************************/

ksensorsSetting::ksensorsSetting()
{
    kscpConfig = KSharedConfig::openConfig();
}

/***************************************************************************
    destructor
 ***************************************************************************/

ksensorsSetting::~ksensorsSetting()
{

}

/***************************************************************************
    get palette
 ***************************************************************************/

void ksensorsSetting::getDefaultPalette (QPalette &pal)
{
    pal.setColor ( QPalette::Base, Qt::white );
    pal.setColor ( QPalette::Window, Qt::white );
    pal.setColor ( QPalette::Light, Qt::white );
    pal.setColor ( QPalette::Dark, Qt::black );
    pal.setColor ( QPalette::WindowText, Qt::black );
    pal.setColor ( QPalette::Text, Qt::black ); 
}

bool ksensorsSetting::getPalette ( QPalette &pal, const QString &name, bool returnDefault )
{
    setGroupConf ( !name.isEmpty() ? name : qsFL1( "General" ) );

    if ( getEntryConf ( qsFL1( "PaletteActive" ), false, ksensorsSetting::AutoDefault ))
    {
        pal.setColor ( QPalette::Base, getEntryConf ( qsFL1( "ColorBackground" ), QColor(), ksensorsSetting::AutoDefault) );
        pal.setColor ( QPalette::Window, getEntryConf ( qsFL1( "ColorBackground" ), QColor(), ksensorsSetting::AutoDefault) );
        pal.setColor ( QPalette::Light, getEntryConf ( qsFL1( "ColorBackground" ), QColor(), ksensorsSetting::AutoDefault) );
        pal.setColor ( QPalette::Dark, getEntryConf ( qsFL1( "ColorShadow" ), QColor(), ksensorsSetting::AutoDefault) );
        pal.setColor ( QPalette::WindowText, getEntryConf ( qsFL1( "ColorValue" ), QColor(), ksensorsSetting::AutoDefault) );
        pal.setColor ( QPalette::Text, getEntryConf ( qsFL1( "ColorTitle" ), QColor(), ksensorsSetting::AutoDefault) );

        return true;
    }
    else
    {
        if (returnDefault)
        {
            getDefaultPalette( pal );

            return true;
        }
    }

    return false;
}

/***************************************************************************
    get update interval
 ***************************************************************************/

double ksensorsSetting::getSettingUpdateInterval ( const QString &key, double dMinimum, bool returnDefault )
{
    double dCheckDefault, value;
    KConfigGroup kcgConfigGen;

    dCheckDefault = -1.0;

    value = getEntryConf ( key, dCheckDefault, ksensorsSetting::Default );

    if ( returnDefault )
    {        
        if ( kcgConfig.name() != qsFL1( "General" ) && ( qFuzzyIsNull ( ( value - dCheckDefault ) ) || qFuzzyIsNull ( value ) ) )
        {
            kcgConfigGen = kscpConfig->group ( qsFL1( "General" ) );
            value = kcgConfigGen.readEntry ( key, dCheckDefault );
        }

        if ( qFuzzyIsNull ( ( value - dCheckDefault ) ) || qFuzzyIsNull ( value ) )
            value = 5.0;
    }

    if ( qFuzzyIsNull ( ( value - dCheckDefault ) ) )
        value = 0.0;

    if ( doubleCompare( value, dMinimum  ) < 0 )
        value = dMinimum;

    return value;
}

/***************************************************************************
    set config group of sensor device
 ***************************************************************************/

void ksensorsSetting::setSensorDevGroupConf ( const QString &shortName, const QString longName, const QString subName )
{
    bool enabShort, enabLong;
    QString qsDefault, qsConfSubName;
    KConfigGroup kcgConfigShort, kcgConfigShortOld;

    enabShort = true;
    enabLong = !longName.isEmpty();

    if ( enabLong )
    {
        kcgConfig = kscpConfig->group ( longName );

        enabShort = !kcgConfig.exists();
    }

    if ( enabShort && enabLong )
    {
        kcgConfigShort = kscpConfig->group ( shortName );

        if ( kcgConfigShort.exists() )
        {
            kcgConfigShort.copyTo ( &kcgConfig );
            kcgConfigShort.deleteGroup();
        }
    }
    
    kcgConfig = kscpConfig->group ( ( enabLong ? longName : shortName ) );

    if ( kcgConfig.exists() )
    {
        qsConfSubName = kcgConfig.readEntry ( "SubName", qsDefault );

        if ( !qsConfSubName.isEmpty() && !subName.isEmpty() && qsConfSubName != subName )
        {
            kcgConfigShortOld = kscpConfig->group ( longName + qsFL1( "_old" ) );
            
            kcgConfig.copyTo ( &kcgConfigShortOld );
            kcgConfig.deleteGroup();
        }
    }
}

/***************************************************************************
    set the parameter
 ***************************************************************************/

void ksensorsSetting::setPar ( QCommandLineParser &parser )
{
    QString optName, parName;
    QStringList qslOptNames;

    qslOptNames = parser.optionNames();

    for ( QStringList::iterator it = qslOptNames.begin(); it != qslOptNames.end(); ++it )    
    {
        parName = optName = *it;
        parName.append( qsFL1( "opt" ) );
        setPar ( parName,  parser.isSet( optName ) );
    }
}

void ksensorsSetting::setPar ( const QString &name,  const bool value )
{
    if ( name == qsFL1( "autostartopt" ) )
        autoStartOpt = value;
    else if ( name == qsFL1( "nodockopt" ) )
        noDockOpt = value;
    else if ( name == qsFL1( "showsplashopt" ) )
        showSplashOpt = value;
    else if ( name == qsFL1( "charttestopt" ) )
        chartTestOpt = value;
}

/***************************************************************************
    get the parameter
 ***************************************************************************/

bool ksensorsSetting::getPar ( const QString &name )
{
    bool value = false;

    if ( name == qsFL1( "autostartopt" ) )
        value = autoStartOpt;
    else if ( name == qsFL1( "nodockopt" ) )
        value = noDockOpt;
    else if ( name == qsFL1( "showsplashopt" ) )
        value = showSplashOpt;
    else if ( name == qsFL1( "charttestopt" ) )
        value = chartTestOpt;

    return value;
}

/***************************************************************************
    set the config group
 ***************************************************************************/

void ksensorsSetting::setGroupConf ( const QString &group )
{
    kcgConfig = kscpConfig->group ( group );
}

/***************************************************************************
    get the config entry value
 ***************************************************************************/

bool ksensorsSetting::getEntryConf ( const QString &key, bool bDefault, Type type )
{
    bool value;
    QStringList qslTrueList, qslFalseList;

    if ( type == Type::AutoDefault )
    {
        qslTrueList << qsFL1( "AutoStart" ) << qsFL1( "showWidget" ) << qsFL1( "windowsSolid" );
        qslFalseList << qsFL1( "show" ) << qsFL1( "showInDock" ) << qsFL1( "panelSolid" ) << qsFL1( "PaletteActive" );

        if ( qslTrueList.indexOf (key) >= 0  )
            bDefault = true;
        else if ( qslFalseList.indexOf (key) >= 0 )
            bDefault = false;
    }

    value =  kcgConfig.readEntry ( key, bDefault );

    if ( kcgConfig.name() == qsFL1( "General" ) )
    {
        if ( key == qsFL1( "showWidget" ) )
        {
            if ( noDockOpt )
                value = true;
        }
    }

    return  value;
} // bool getEntryConf ( const QString &key, bool bDefault, Type type )

int ksensorsSetting::getEntryConf ( const QString &key, int iDefault, Type type )
{
    int value;

    if ( type == Type::AutoDefault )
    {
        if ( key == qsFL1( "desktop" ) )
            iDefault = 0;
        else if ( key == qsFL1( "Alarm" ) )
            iDefault = Actions::acNothing;
        else if ( key == qsFL1( "PanelsSize" ) )
            iDefault = 64;
    }

    value = kcgConfig.readEntry ( key, iDefault );

//  if ( kcgConfig.name() == "General" )
    {
        if ( key == qsFL1( "desktop" ) )
        {
            if ( noDockOpt )
                value = 0;
        }
    }

    return  value;
} // int getEntryConf ( const QString &key, int iDefault, Type type )

double ksensorsSetting::getEntryConf ( const QString &key, double dDefault, Type type )
{
    double value;
    QString qsValue;
    
    if ( type == Type::AutoDefault )
    {
        if ( key == qsFL1( "UpdateInterval" ) )
            dDefault = 5.0;
        else if ( key == qsFL1( "panelOpacityValue" ) || key == qsFL1( "windowsOpacityValue" ) )
            dDefault = 100.0;
    }

    value = kcgConfig.readEntry ( key, dDefault );

//  if ( kcgConfig.name() == "ShowWidget" )
    {
        if ( key == qsFL1( "panelOpacityValue" ) )
        {
            qsValue = getEntryConf ( qsFL1( "panelSolid" ), qsFL1( "empty" ), Type::Default );

            if ( qsValue == qsFL1( "empty" ) )
            {
                if ( getEntryConf ( qsFL1( "panelSolid" ), false, Type::AutoDefault ) )
                    value = 100.0;
                else
                    value = 70.0;
            }
        }
        else if ( key == qsFL1( "windowsOpacityValue" ) )
        {
            qsValue = getEntryConf ( qsFL1( "windowsSolid" ), qsFL1( "empty" ), Type::Default );

            if ( qsValue == qsFL1( "empty" ) )
            {
                if ( getEntryConf ( qsFL1( "windowsSolid" ), false, Type::AutoDefault ) )
                    value = 100.0;
                else
                    value = 70.0;
            }
        }
    }

    return  value;
} // double getEntryConf ( const QString &key, double dDefault, Type type );

QString ksensorsSetting::getEntryConf ( const QString &key, QString qsDefault, Type type )
{
    if ( type == Type::AutoDefault )
    {
        if ( key == qsFL1( "AlarmSound" ) )
            qsDefault = qsFL1( "ksensors_alert.wav" );
        else if ( key == qsFL1( "Scale" ) )
            qsDefault = qsFL1( "C" );
    }

    return  ( kcgConfig.readEntry ( key, qsDefault ) );
} // QString getEntryConf ( const QString &key, QString qsDefault, Type type )

QColor ksensorsSetting::getEntryConf ( const QString &key, QColor qcDefault, Type type )
{
    if ( type == Type::AutoDefault )
    {
        if ( key == qsFL1( "colorNormal" ) )
            qcDefault = QColor ( Qt::darkGreen );
        else if ( key == qsFL1( "colorAlarm" ) )
            qcDefault = QColor ( Qt::red );
        else if ( key == qsFL1( "ColorBackground" ) )
            qcDefault = QColor ( Qt::white );
        else if ( key == qsFL1( "ColorShadow" ) || key == qsFL1( "ColorValue" ) || key == qsFL1( "ColorTitle" ) )
            qcDefault = QColor ( Qt::black );
    }

    return  ( kcgConfig.readEntry ( key, qcDefault ) );
} // QColor getEntryConf ( const QString &key, QColor qcDefault, Type type )


QPoint ksensorsSetting::getEntryConf ( const QString &key, QPoint qpDefault, Type type )
{
    if ( type == Type::AutoDefault )
    {
        if ( key == qsFL1( "showPos" ) )
            qpDefault = QPoint (-1, -1);
    }

    return  ( kcgConfig.readEntry ( key, qpDefault ) );
} // QPoint getEntryConf ( const QString &key, QPoint qpDefault, Type type )

QRect ksensorsSetting::getEntryConf ( const QString &key, QRect qrDefault, Type type )
{
    if ( type == Type::AutoDefault )
    {
        if ( key == qsFL1( "geometry" ) )
            qrDefault = QRect ( 32, 32, 64*4, 64*3 );
    }

    return  ( kcgConfig.readEntry ( key, qrDefault ) );
} // QRect getEntryConf ( const QString &key, QRect qrDefault, Type type )

/***************************************************************************
    set the config entry value
 ***************************************************************************/

bool ksensorsSetting::setEntryConf ( const QString &key, const bool value )
{
    bool withChange;

    withChange = value != kcgConfig.readEntry ( key, value );

    kcgConfig.writeEntry ( key, value );

    return withChange;
}

bool ksensorsSetting::setEntryConf ( const QString &key, const int value )
{
    bool withChange;

    withChange = value != kcgConfig.readEntry ( key, value );

    kcgConfig.writeEntry ( key, value );

    return withChange;
}

bool ksensorsSetting::setEntryConf ( const QString &key, const double value, double dDefault, Type type )
{
    bool withChange;

    withChange = value != kcgConfig.readEntry ( key, value );

    if ( type == Type::DeleteDefault && value == dDefault )
        kcgConfig.deleteEntry ( key );
    else    
        kcgConfig.writeEntry ( key, value );

    return withChange;
}

bool ksensorsSetting::setEntryConf ( const QString &key, const QString &value )
{
    bool withChange;

    withChange = value != kcgConfig.readEntry ( key, value );

    kcgConfig.writeEntry ( key, value );

    return withChange;
}

bool ksensorsSetting::setEntryConf ( const QString &key, const QColor &value )
{
    bool withChange;

    withChange = value != kcgConfig.readEntry ( key, value );

    kcgConfig.writeEntry ( key, value );

    return withChange;
}

bool ksensorsSetting::setEntryConf ( const QString &key, const QPoint &value )
{
    bool withChange;

    withChange = value != kcgConfig.readEntry ( key, value );

    kcgConfig.writeEntry ( key, value );

    return withChange;
}

bool ksensorsSetting::setEntryConf ( const QString &key, const QRect &value )
{
    bool withChange;

    withChange = value != kcgConfig.readEntry ( key, value );

    kcgConfig.writeEntry ( key, value );

    return withChange;
}

/***************************************************************************
    sync the configuration
 ***************************************************************************/

bool ksensorsSetting::syncConf()
{
    return ( kscpConfig->sync() );
}
